@@ -1,12 +1,17 @@
-from _root_numpy import (
+from _tree import (
     root2array, root2rec,
     tree2array, tree2rec,
     array2tree, array2root,
-    list_trees, list_branches, list_structures,
-    fill_array, fill_hist,
-    random_sample, array, matrix)
+    list_trees, list_branches, list_structures)
+from _hist import fill_hist
+from _graph import fill_graph
+from _sample import random_sample
+from _array import array
+from _matrix import matrix
 from _warnings import RootNumpyWarning, RootNumpyUnconvertibleWarning
-from utils import stretch, blockwise_inner_join
+from _utils import (
+    stretch, blockwise_inner_join,
+    rec2array, stack, dup_idx)
 from info import __version__
 
 __all__ = [
@@ -16,15 +21,18 @@ __all__ = [
     'tree2rec',
     'array2tree',
     'array2root',
-    'fill_array',
     'fill_hist',
+    'fill_graph',
     'random_sample',
     'array',
     'matrix',
     'list_trees',
     'list_branches',
     'list_structures',
+    'rec2array',
+    'stack',
     'stretch',
+    'dup_idx',
     'blockwise_inner_join',
     'RootNumpyWarning',
     'RootNumpyUnconvertibleWarning',
